@@ -12,10 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from keras_nlp.tokenizers.byte_pair_tokenizer import BytePairTokenizer
 from keras_nlp.tokenizers.byte_tokenizer import ByteTokenizer
 from keras_nlp.tokenizers.sentence_piece_tokenizer import SentencePieceTokenizer
+from keras_nlp.tokenizers.sentence_piece_tokenizer_trainer import (
+    compute_sentence_piece_proto,
+)
 from keras_nlp.tokenizers.tokenizer import Tokenizer
-from keras_nlp.tokenizers.unicode_character_tokenizer import (
-    UnicodeCharacterTokenizer,
+from keras_nlp.tokenizers.unicode_codepoint_tokenizer import (
+    UnicodeCodepointTokenizer,
 )
 from keras_nlp.tokenizers.word_piece_tokenizer import WordPieceTokenizer
+from keras_nlp.tokenizers.word_piece_tokenizer_trainer import (
+    compute_word_piece_vocabulary,
+)
