@@ -16,11 +16,18 @@
 
 import numpy as np
 import tensorflow as tf
-import tensorflow_text as tf_text
+from tensorflow import keras
 
 from keras_nlp.tokenizers import tokenizer
+from keras_nlp.utils.tf_utils import assert_tf_text_installed
+
+try:
+    import tensorflow_text as tf_text
+except ImportError:
+    tf_text = None
 
 
+@keras.utils.register_keras_serializable(package="keras_nlp")
 class ByteTokenizer(tokenizer.Tokenizer):
     """Raw byte tokenizer.
 
@@ -54,10 +61,15 @@ class ByteTokenizer(tokenizer.Tokenizer):
         normalization_form: string. One of the following values: (None, "NFC",
             "NFKC", "NFD", "NFKD"). If set, every UTF-8 string in the input
             tensor text will be normalized to the given form before tokenizing.
-        errors: string. One of ("strict", "replace", "ignore"). Defaults to
-            "replace". Specifies the `detokenize()` behaviour when an invalid
-            byte sequence is encountered (same behaviour as
-            https://www.tensorflow.org/api_docs/python/tf/strings/unicode_transcode).
+        errors: One of ('replace', 'remove', 'strict'). Specifies the
+            `detokenize()` behavior when an invalid tokenizer is encountered.
+            The value of `'strict'` will cause the operation to produce a
+            `InvalidArgument` error on any invalid input formatting. A value of
+            `'replace'` will cause the tokenizer to replace any invalid
+            formatting in the input with the `replacement_char` codepoint.
+            A value of `'ignore'` will cause the tokenizer to skip any invalid
+            formatting in the input and produce no corresponding output
+            character.
         replacement_char: int. Defaults to 65533. The replacement character to
             use when an invalid byte sequence is encountered and when `errors`
             is set to "replace" (same behaviour as
@@ -150,6 +162,8 @@ class ByteTokenizer(tokenizer.Tokenizer):
         replacement_char: int = 65533,
         **kwargs,
     ):
+        assert_tf_text_installed(self.__class__.__name__)
+
         # Check dtype and provide a default.
         if "dtype" not in kwargs or kwargs["dtype"] is None:
             kwargs["dtype"] = tf.int32
