@@ -15,26 +15,19 @@
 """Text generation utilities."""
 
 import tensorflow as tf
+from absl import logging
+from tensorflow import keras
 
 
-def validate_prompt(prompt):
+def _validate_prompt(prompt):
     """Helper function to validate input to text_generation utils."""
-    if isinstance(prompt, tf.RaggedTensor):
-        raise ValueError(
-            "RaggedTensor `prompt` is not supported, please "
-            "provide `prompt` as a list or Tensor."
-        )
-    if not isinstance(prompt, tf.Tensor):
+    if not isinstance(prompt, (tf.Tensor, tf.RaggedTensor)):
         prompt = tf.convert_to_tensor(prompt)
-    if prompt.shape[-1] == 0:
-        raise ValueError(
-            "Length of `prompt` is 0, please provide a non-empty `prompt`."
-        )
     return prompt
 
 
-def validate_token_probability_fn(token_probability_fn, prompt):
-    """Helper function to validate token probability fn output"""
+def _validate_token_probability_fn(token_probability_fn, prompt):
+    """Helper function to validate `token_probability_fn` output."""
     test_pred = token_probability_fn(prompt)
     if len(test_pred.shape) != 2:
         raise ValueError(
@@ -43,14 +36,59 @@ def validate_token_probability_fn(token_probability_fn, prompt):
             "[batch_size, vocab_size]."
         )
 
+    return tf.shape(test_pred)[-1], test_pred.dtype
 
-def mask_tokens_after_end_token(prompt, max_length, end_token_id, pad_token_id):
+
+def _get_prompt_shape(prompt):
+    """Helper function to get the batch size and prompt length."""
+    if isinstance(prompt, tf.Tensor):
+        shape = tf.shape(prompt)
+        return (shape[0], shape[1])
+    elif isinstance(prompt, tf.RaggedTensor):
+        batch_size = prompt.nrows()
+        length = tf.math.reduce_min(tf.RaggedTensor.row_lengths(prompt))
+        return (batch_size, length)
+
+
+def _pad_prompt(prompt, max_length):
+    """Pad prompt to `max_length` and compute a mask for controlled updates.
+
+    This utility will pad the (possibly ragged) prompt to `max_length`, and
+    compute a mask where the input was originally set in the prompt, to avoid
+    overwriting the original inputs when generating token(s) for the next
+    timestep.
+    """
+    if isinstance(prompt, tf.Tensor):
+        shape = tf.shape(prompt)
+        extra_space = tf.math.maximum(0, max_length - shape[1])
+        pad_shape = [shape[0], extra_space]
+
+        mask = tf.ones(shape, tf.bool)
+        mask = tf.concat((mask, tf.zeros(pad_shape, tf.bool)), axis=1)
+        prompt = tf.concat((prompt, tf.zeros(pad_shape, prompt.dtype)), axis=1)
+    elif isinstance(prompt, tf.RaggedTensor):
+        # TODO: `to_tensor()` works with `jit_compile = True` in TF 2.8.x but
+        # fails in TF 2.9.x. Fix this. After this issue has been fixed, we can
+        # condense the two branches into one by starting off with a ragged tensor.
+        mask = tf.ones_like(prompt, dtype=tf.bool)
+        mask = mask.to_tensor(shape=(None, max_length))
+        prompt = prompt.to_tensor(shape=(None, max_length))
+    return prompt, mask
+
+
+def _mask_tokens_after_end_token(
+    prompt, max_length, end_token_id, pad_token_id
+):
     """Helper function to mask the tokens after the end token."""
     # Mask out tokens after `end_token_id` is encountered.
     # Find index of first end_token_id.
     end_indices = tf.math.argmax(prompt == end_token_id, -1)
     # Use max_length if no `end_token_id` is found.
-    end_indices = tf.where(end_indices == 0, max_length, end_indices)
+    end_indices = tf.where(
+        end_indices == 0,
+        tf.cast(max_length, dtype=end_indices.dtype),
+        end_indices,
+    )
     # Build a mask including end_token and replace tokens after end_token
     # with `pad_token_id`.
     valid_indices = tf.sequence_mask(end_indices + 1, maxlen=max_length)
@@ -64,8 +102,7 @@ def greedy_search(
     end_token_id=None,
     pad_token_id=0,
 ):
-    """
-    Text generation utility based on greedy search.
+    """Text generation utility based on greedy search.
 
     Greedy search always appends the token having the largest probability to
     existing sequence.
@@ -98,14 +135,14 @@ def greedy_search(
     END_ID = 2
 
     # Create a dummy model to predict the next token.
-    model = tf.keras.Sequential(
+    model = keras.Sequential(
         [
-            tf.keras.Input(shape=[None]),
-            tf.keras.layers.Embedding(
+            keras.Input(shape=[None]),
+            keras.layers.Embedding(
                 input_dim=VOCAB_SIZE,
                 output_dim=FEATURE_SIZE,
             ),
-            tf.keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
+            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
         ]
     )
 
@@ -126,38 +163,51 @@ def greedy_search(
     ```
 
     """
-    if not tf.executing_eagerly():
-        raise RuntimeError(
-            "`keras_nlp.utils.greedy_search` currently requires an eager "
-            "execution context. Please call `greedy_search` outside "
-            "tf.function or run `tf.config.run_functions_eagerly(True)` to run "
-            "tf.function in eager mode."
-        )
-
-    prompt = validate_prompt(prompt)
+    prompt = _validate_prompt(prompt)
 
     input_is_1d = prompt.shape.rank == 1
     if input_is_1d:
         prompt = prompt[tf.newaxis, :]
-    validate_token_probability_fn(token_probability_fn, prompt)
 
-    i = prompt.shape[1]
-    while i < max_length:
-        # If the prompt has reached our desired length, exit while loop.
-        pred = token_probability_fn(prompt)
+    batch_size, length = _get_prompt_shape(prompt)
+    prompt, mask = _pad_prompt(prompt, max_length)
+
+    _validate_token_probability_fn(token_probability_fn, prompt)
+
+    def one_step(length, prompt):
+        pred = token_probability_fn(prompt[:, :length])
         next_token = tf.cast(tf.argmax(pred, axis=-1), dtype=prompt.dtype)
+        next_token = tf.where(mask[:, length], prompt[:, length], next_token)
+
         # Append the next token to current sequence.
-        prompt = tf.concat([prompt, next_token[:, tf.newaxis]], axis=-1)
-        i += 1
+        prompt = tf.tensor_scatter_nd_update(
+            tensor=prompt,
+            indices=tf.stack(
+                (
+                    tf.cast(tf.range(batch_size), dtype=length.dtype),
+                    tf.repeat(length, batch_size),
+                ),
+                axis=1,
+            ),
+            updates=next_token,
+        )
+
+        length = tf.add(length, 1)
+        return (length, prompt)
+
+    # Run a while loop till text of length `max_length` has been generated.
+    length, prompt = tf.while_loop(
+        cond=lambda length, _: tf.less(length, max_length),
+        body=one_step,
+        loop_vars=(length, prompt),
+    )
 
     if end_token_id is not None:
-        prompt = mask_tokens_after_end_token(
+        prompt = _mask_tokens_after_end_token(
             prompt, max_length, end_token_id, pad_token_id
         )
 
-    if input_is_1d:
-        return tf.squeeze(prompt)
-    return prompt
+    return tf.squeeze(prompt) if input_is_1d else prompt
 
 
 def beam_search(
@@ -169,8 +219,7 @@ def beam_search(
     end_token_id=None,
     pad_token_id=0,
 ):
-    """
-    Text generation utility based on beam search algorithm.
+    """Text generation utility based on beam search algorithm.
 
     At each time-step, beam search keeps the beams (sequences) of the top
     `num_beams` highest accumulated probabilities, and uses each one of the
@@ -178,11 +227,10 @@ def beam_search(
 
     Args:
         token_probability_fn: a callable, which takes in input_sequence
-            and output the probability distribution of the next token. If
+            and outputs the probability distribution of the next token. If
             `from_logits` set to True, it should output the logits of the next
-            token. The input shape would be `[batch_size, length]` and the
-            output should be `[batch_size, vocab_size]`, where batch_size is
-            variable.
+            token. The input shape would be `[batch_size * num_beams, length]`
+            and the output should be `[batch_size * num_beams, vocab_size]`.
         prompt: a list or a Tensor, can be 1D or 2D, the initial tokens to
             append generated tokens. The initial beam for beam search.
         max_length: int. The max length of generated text.
@@ -211,14 +259,14 @@ def beam_search(
     END_ID = 2
 
     # Create a dummy model to predict the next token.
-    model = tf.keras.Sequential(
+    model = keras.Sequential(
         [
-            tf.keras.Input(shape=[None]),
-            tf.keras.layers.Embedding(
+            keras.Input(shape=[None]),
+            keras.layers.Embedding(
                 input_dim=VOCAB_SIZE,
                 output_dim=FEATURE_SIZE,
             ),
-            tf.keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
+            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
         ]
     )
 
@@ -240,74 +288,116 @@ def beam_search(
     ```
 
     """
-    if not tf.executing_eagerly():
-        raise RuntimeError(
-            "`keras_nlp.utils.beam_search` currently requires an eager "
-            "execution context. Please call `beam_search` outside "
-            "tf.function or run `tf.config.run_functions_eagerly(True)` to run "
-            "tf.function in eager mode."
-        )
     if num_beams <= 0:
         raise ValueError(
             f"`num_beams` should be strictly positive. Received: `num_beams={num_beams}`."
         )
+    if num_beams == 1:
+        return greedy_search(
+            token_probability_fn=token_probability_fn,
+            prompt=prompt,
+            max_length=max_length,
+            end_token_id=end_token_id,
+            pad_token_id=pad_token_id,
+        )
 
-    prompt = validate_prompt(prompt)
+    prompt = _validate_prompt(prompt)
 
     input_is_1d = prompt.shape.rank == 1
     if input_is_1d:
         prompt = prompt[tf.newaxis, :]
-    validate_token_probability_fn(token_probability_fn, prompt)
 
-    batch_size, length = prompt.shape
-    if length < max_length:
-        # Initialize beam.
-        beams = tf.expand_dims(prompt, 1)
-        beams_prob = tf.zeros([batch_size, 1])
-        i = length
-        while i < max_length:
-            beam_size = beams.shape[1]
-            beam_preds = []
-            for j in range(beam_size):
-                preds = token_probability_fn(beams[:, j, :])
-                if from_logits:
-                    preds = tf.keras.activations.softmax(preds, axis=-1)
-                beam_preds.append(preds)
-            stacked_preds = tf.stack(beam_preds, axis=1)
-            vocab_size = stacked_preds.shape[2]
-            logits = tf.reshape(
-                stacked_preds, [batch_size, beam_size * vocab_size]
-            )
-            probs = tf.math.log(logits) + tf.repeat(
-                beams_prob, repeats=vocab_size, axis=1
-            )
-            num_beams = min(beam_size * vocab_size, num_beams)
-            candidate_prob, candidate_indexes = tf.math.top_k(
-                probs, k=num_beams, sorted=False
-            )
-            candidate_beam_indexes = candidate_indexes // vocab_size
-            next_token = candidate_indexes % vocab_size
+    batch_size, length = _get_prompt_shape(prompt)
+    prompt, mask = _pad_prompt(prompt, max_length)
 
-            beams = tf.gather(
-                beams, candidate_beam_indexes, axis=1, batch_dims=1
-            )
-            beams = tf.concat([beams, next_token[..., tf.newaxis]], axis=-1)
-            beams_prob = candidate_prob
-            i += 1
-        # Get the beam with the maximum probability.
-        max_indexes = tf.math.argmax(beams_prob, axis=-1)
-        max_beams = tf.gather(
-            beams, max_indexes[:, tf.newaxis], axis=1, batch_dims=1
+    vocab_size, pred_dtype = _validate_token_probability_fn(
+        token_probability_fn, prompt
+    )
+
+    if length >= max_length:
+        return tf.squeeze(prompt) if input_is_1d else prompt
+
+    # Initialize beam with shape `(batch_size, num_beams, length)`.
+    beams = tf.repeat(tf.expand_dims(prompt, axis=1), num_beams, axis=1)
+
+    # Initialize `beams_prob` with shape `(batch_size, num_beams)`.
+    beams_prob = tf.zeros([batch_size, 1], dtype=pred_dtype)
+    beams_prob = tf.concat(
+        [beams_prob, tf.fill((batch_size, num_beams - 1), pred_dtype.min)],
+        axis=-1,
+    )
+
+    def one_step(beams, beams_prob, length):
+        truncated_beams = beams[..., :length]
+
+        flattened_beams = tf.reshape(
+            truncated_beams, shape=[batch_size * num_beams, -1]
         )
-        prompt = tf.squeeze(max_beams)
+        preds = token_probability_fn(flattened_beams)
+        if from_logits:
+            preds = keras.activations.softmax(preds, axis=-1)
+        # Reshape `preds` to shape `(batch_size, num_beams * vocab_size)`.
+        preds = tf.reshape(preds, shape=[batch_size, -1])
+
+        probs = tf.math.log(preds) + tf.repeat(
+            beams_prob, repeats=vocab_size, axis=1
+        )
+
+        candidate_prob, candidate_indexes = tf.math.top_k(
+            probs, k=num_beams, sorted=False
+        )
+        candidate_beam_indexes = candidate_indexes // vocab_size
+        next_token = candidate_indexes % vocab_size
+
+        beams = tf.gather(beams, candidate_beam_indexes, axis=1, batch_dims=1)
+
+        # Build a new column of updates to scatter into the beam tensor.
+        next_token = tf.where(
+            condition=mask[..., length, tf.newaxis],
+            x=beams[..., length],
+            y=next_token,
+        )
+        next_token = tf.reshape(next_token, shape=[-1])
+
+        # Generate `(batch_index, beam_index)` tuples for each beam.
+        beam_indices = tf.where(tf.ones((batch_size, num_beams), tf.bool))
+        beam_indices = tf.cast(beam_indices, dtype=length.dtype)
+        # Build a tensor of repeated `length` values.
+        length_indices = tf.fill((batch_size * num_beams, 1), length)
+        # Concatenate to a triplet of `(batch_index, beam_index, length)`.
+        indices = tf.concat([beam_indices, length_indices], axis=-1)
+
+        # Update `beams[:, :, length]` with `next_token`.
+        beams = tf.tensor_scatter_nd_update(
+            tensor=beams,
+            indices=indices,
+            updates=next_token,
+        )
+
+        beams_prob = candidate_prob
+        length = tf.add(length, 1)
+
+        return beams, beams_prob, length
+
+    # Run a while loop till text of length `max_length` has been generated.
+    beams, beams_prob, length = tf.while_loop(
+        cond=lambda beams, beams_prob, length: tf.less(length, max_length),
+        body=one_step,
+        loop_vars=(beams, beams_prob, length),
+    )
+
+    # Get the beam with the maximum probability.
+    max_indexes = tf.math.argmax(beams_prob, axis=-1)
+    max_beams = tf.gather(
+        beams, max_indexes[:, tf.newaxis], axis=1, batch_dims=1
+    )
+    prompt = tf.squeeze(max_beams)
 
     if end_token_id is not None:
-        prompt = mask_tokens_after_end_token(
+        prompt = _mask_tokens_after_end_token(
             prompt, max_length, end_token_id, pad_token_id
         )
-    if input_is_1d:
-        return tf.squeeze(prompt)
-    return prompt
+    return tf.squeeze(prompt) if input_is_1d else prompt
 
 
 def random_search(
@@ -319,8 +409,7 @@ def random_search(
     end_token_id=None,
     pad_token_id=0,
 ):
-    """
-    Text generation utility based on randomly sampling the entire probability
+    """Text generation utility based on randomly sampling the entire probability
     distribution.
 
     Random sampling samples the next token from the probability distribution
@@ -358,14 +447,14 @@ def random_search(
     END_ID = 2
 
     # Create a dummy model to predict the next token.
-    model = tf.keras.Sequential(
+    model = keras.Sequential(
         [
-            tf.keras.Input(shape=[None]),
-            tf.keras.layers.Embedding(
+            keras.Input(shape=[None]),
+            keras.layers.Embedding(
                 input_dim=VOCAB_SIZE,
                 output_dim=FEATURE_SIZE,
             ),
-            tf.keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
+            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
         ]
     )
 
@@ -386,41 +475,59 @@ def random_search(
     ```
 
     """
-    if not tf.executing_eagerly():
-        raise RuntimeError(
-            "`keras_nlp.utils.random_sampling` currently requires an eager "
-            "execution context. Please call `random_sampling` outside "
-            "tf.function or run `tf.config.run_functions_eagerly(True)` to run "
-            "tf.function in eager mode."
-        )
+    prompt = _validate_prompt(prompt)
 
-    prompt = validate_prompt(prompt)
     input_is_1d = prompt.shape.rank == 1
     if input_is_1d:
         prompt = prompt[tf.newaxis, :]
-    validate_token_probability_fn(token_probability_fn, prompt)
 
-    i = prompt.shape[1]
-    while i < max_length:
-        # If the prompt has reached our desired length, exit while loop.
-        pred = token_probability_fn(prompt)
+    batch_size, length = _get_prompt_shape(prompt)
+    prompt, mask = _pad_prompt(prompt, max_length)
+
+    _validate_token_probability_fn(token_probability_fn, prompt)
+
+    def one_step(length, prompt):
+        pred = token_probability_fn(prompt[:, :length])
         if from_logits:
-            pred = tf.keras.activations.softmax(pred, axis=-1)
-        next_token = tf.cast(
-            tf.random.categorical(tf.math.log(pred), 1, seed=seed),
-            dtype=prompt.dtype,
+            pred = keras.activations.softmax(pred, axis=-1)
+        next_token = tf.squeeze(
+            tf.cast(
+                tf.random.categorical(tf.math.log(pred), 1, seed=seed),
+                dtype=prompt.dtype,
+            ),
+            axis=1,
         )
+        next_token = tf.where(mask[:, length], prompt[:, length], next_token)
+
         # Append the next token to current sequence.
-        prompt = tf.concat([prompt, next_token], axis=-1)
-        i += 1
+        prompt = tf.tensor_scatter_nd_update(
+            tensor=prompt,
+            indices=tf.stack(
+                (
+                    tf.cast(tf.range(batch_size), dtype=length.dtype),
+                    tf.repeat(length, batch_size),
+                ),
+                axis=1,
+            ),
+            updates=next_token,
+        )
+
+        length = tf.add(length, 1)
+        return (length, prompt)
+
+    # Run a while loop till text of length `max_length` has been generated.
+    length, prompt = tf.while_loop(
+        cond=lambda length, _: tf.less(length, max_length),
+        body=one_step,
+        loop_vars=(length, prompt),
+    )
 
     if end_token_id is not None:
-        prompt = mask_tokens_after_end_token(
+        prompt = _mask_tokens_after_end_token(
             prompt, max_length, end_token_id, pad_token_id
         )
-    if input_is_1d:
-        return tf.squeeze(prompt)
-    return prompt
+
+    return tf.squeeze(prompt) if input_is_1d else prompt
 
 
 def top_k_search(
@@ -433,8 +540,7 @@ def top_k_search(
     end_token_id=None,
     pad_token_id=0,
 ):
-    """
-    Text generation utility based on top-k sampling.
+    """Text generation utility based on top-k sampling.
 
     Top-k search samples the next token from the top-k tokens in the
     probability distribution provided by `token_probability_fn` and appends it
@@ -474,14 +580,14 @@ def top_k_search(
     END_ID = 2
 
     # Create a dummy model to predict the next token.
-    model = tf.keras.Sequential(
+    model = keras.Sequential(
         [
-            tf.keras.Input(shape=[None]),
-            tf.keras.layers.Embedding(
+            keras.Input(shape=[None]),
+            keras.layers.Embedding(
                 input_dim=VOCAB_SIZE,
                 output_dim=FEATURE_SIZE,
             ),
-            tf.keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
+            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
         ]
     )
 
@@ -503,50 +609,75 @@ def top_k_search(
     ```
 
     """
-    if not tf.executing_eagerly():
-        raise RuntimeError(
-            "`keras_nlp.utils.top_k_search` currently requires an eager "
-            "execution context. Please call `top_k_search` outside "
-            "tf.function or run `tf.config.run_functions_eagerly(True)` to run "
-            "tf.function in eager mode."
-        )
     if k <= 0:
         raise ValueError(f"`k` should be strictly positive. Received: `k={k}`.")
 
-    prompt = validate_prompt(prompt)
+    prompt = _validate_prompt(prompt)
+
     input_is_1d = prompt.shape.rank == 1
     if input_is_1d:
         prompt = prompt[tf.newaxis, :]
-    validate_token_probability_fn(token_probability_fn, prompt)
 
-    i = prompt.shape[1]
-    while i < max_length:
-        # If the prompt has reached our desired length, exit while loop.
-        pred = token_probability_fn(prompt)
+    batch_size, length = _get_prompt_shape(prompt)
+    prompt, mask = _pad_prompt(prompt, max_length)
+
+    _validate_token_probability_fn(token_probability_fn, prompt)
+
+    # If k is greater than the vocabulary size, use the entire vocabulary.
+    pred = token_probability_fn(prompt)
+    if k > pred.shape[1]:
+        logging.warning(
+            f"`k` larger than vocabulary size={pred.shape[1]}."
+            f"Setting `k` to vocabulary size. Received: `k={k}`."
+        )
+        k = pred.shape[1]
+
+    def one_step(length, prompt):
+        pred = token_probability_fn(prompt[:, :length])
         if from_logits:
-            pred = tf.keras.activations.softmax(pred, axis=-1)
-        # If k is greater than the vocabulary size, use the entire vocabulary.
-        k = min(k, pred.shape[1])
+            pred = keras.activations.softmax(pred, axis=-1)
+
         # Filter out top-k tokens.
         top_k_pred, top_k_indices = tf.math.top_k(pred, k=k, sorted=False)
         # Sample the next token from the probability distribution.
         next_token = tf.random.categorical(
             tf.math.log(top_k_pred), 1, seed=seed
         )
+
         # Rearrange to get the next token idx from the original order.
         next_token = tf.gather_nd(top_k_indices, next_token, batch_dims=1)
         next_token = tf.cast(next_token, dtype=prompt.dtype)
+        next_token = tf.where(mask[:, length], prompt[:, length], next_token)
+
         # Append the next token to current sequence.
-        prompt = tf.concat([prompt, next_token[:, tf.newaxis]], axis=-1)
-        i += 1
+        prompt = tf.tensor_scatter_nd_update(
+            tensor=prompt,
+            indices=tf.stack(
+                (
+                    tf.cast(tf.range(batch_size), dtype=length.dtype),
+                    tf.repeat(length, batch_size),
+                ),
+                axis=1,
+            ),
+            updates=next_token,
+        )
+
+        length = tf.add(length, 1)
+        return (length, prompt)
+
+    # Run a while loop till text of length `max_length` has been generated.
+    length, prompt = tf.while_loop(
+        cond=lambda length, _: tf.less(length, max_length),
+        body=one_step,
+        loop_vars=(length, prompt),
+    )
 
     if end_token_id is not None:
-        prompt = mask_tokens_after_end_token(
+        prompt = _mask_tokens_after_end_token(
             prompt, max_length, end_token_id, pad_token_id
         )
-    if input_is_1d:
-        return tf.squeeze(prompt)
-    return prompt
+
+    return tf.squeeze(prompt) if input_is_1d else prompt
 
 
 def top_p_search(
@@ -559,8 +690,7 @@ def top_p_search(
     end_token_id=None,
     pad_token_id=0,
 ):
-    """
-    Text generation utility based on top-p (nucleus) sampling.
+    """Text generation utility based on top-p (nucleus) sampling.
 
     Top-p search selects tokens from the smallest subset of output probabilities
     that sum to greater than `p`. Put another way, top-p will first order
@@ -602,14 +732,14 @@ def top_p_search(
     END_ID = 2
 
     # Create a dummy model to predict the next token.
-    model = tf.keras.Sequential(
+    model = keras.Sequential(
         [
-            tf.keras.Input(shape=[None]),
-            tf.keras.layers.Embedding(
+            keras.Input(shape=[None]),
+            keras.layers.Embedding(
                 input_dim=VOCAB_SIZE,
                 output_dim=FEATURE_SIZE,
             ),
-            tf.keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
+            keras.layers.Dense(VOCAB_SIZE, activation="softmax"),
         ]
     )
 
@@ -631,30 +761,26 @@ def top_p_search(
     ```
 
     """
-    if not tf.executing_eagerly():
-        raise RuntimeError(
-            "`keras_nlp.utils.top_p_search` currently requires an eager "
-            "execution context. Please call `top_p_search` outside "
-            "tf.function or run `tf.config.run_functions_eagerly(True)` to run "
-            "tf.function in eager mode."
-        )
     if p <= 0 or p >= 1:
         raise ValueError(
             f"`p` should be in the range (0, 1). Received: `p={p}`."
         )
 
-    prompt = validate_prompt(prompt)
+    prompt = _validate_prompt(prompt)
+
     input_is_1d = prompt.shape.rank == 1
     if input_is_1d:
         prompt = prompt[tf.newaxis, :]
-    validate_token_probability_fn(token_probability_fn, prompt)
 
-    i = prompt.shape[1]
-    while i < max_length:
-        # If the prompt has reached our desired length, exit while loop.
-        pred = token_probability_fn(prompt)
+    batch_size, length = _get_prompt_shape(prompt)
+    prompt, mask = _pad_prompt(prompt, max_length)
+
+    _validate_token_probability_fn(token_probability_fn, prompt)
+
+    def one_step(length, prompt):
+        pred = token_probability_fn(prompt[:, :length])
         if from_logits:
-            pred = tf.keras.activations.softmax(pred, axis=-1)
+            pred = keras.activations.softmax(pred, axis=-1)
         # Sort preds in descending order.
         sorted_preds, sorted_indices = tf.math.top_k(
             pred, k=pred.shape[1], sorted=True
@@ -671,7 +797,7 @@ def top_p_search(
         probs = tf.where(
             shifted_keep_mask,
             sorted_preds,
-            tf.zeros(pred.shape, dtype=sorted_preds.dtype),
+            tf.zeros(tf.shape(pred), dtype=sorted_preds.dtype),
         )
         sorted_next_token = tf.random.categorical(
             tf.math.log(probs), 1, seed=seed
@@ -680,14 +806,34 @@ def top_p_search(
             sorted_indices, sorted_next_token, batch_dims=1
         )
         next_token = tf.cast(next_token, dtype=prompt.dtype)
+        next_token = tf.where(mask[:, length], prompt[:, length], next_token)
+
         # Append the next token to current sequence.
-        prompt = tf.concat([prompt, next_token[:, tf.newaxis]], axis=-1)
-        i += 1
+        prompt = tf.tensor_scatter_nd_update(
+            tensor=prompt,
+            indices=tf.stack(
+                (
+                    tf.cast(tf.range(batch_size), dtype=length.dtype),
+                    tf.repeat(length, batch_size),
+                ),
+                axis=1,
+            ),
+            updates=next_token,
+        )
+
+        length = tf.add(length, 1)
+        return (length, prompt)
+
+    # Run a while loop till text of length `max_length` has been generated.
+    length, prompt = tf.while_loop(
+        cond=lambda length, _: tf.less(length, max_length),
+        body=one_step,
+        loop_vars=(length, prompt),
+    )
 
     if end_token_id is not None:
-        prompt = mask_tokens_after_end_token(
+        prompt = _mask_tokens_after_end_token(
             prompt, max_length, end_token_id, pad_token_id
         )
-    if input_is_1d:
-        return tf.squeeze(prompt)
-    return prompt
+
+    return tf.squeeze(prompt) if input_is_1d else prompt
